/*
 CPLD Logic Analyser
 Version: 0.2
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/12/2013
 Last Modified: 10/07/2014
 
 This project uses an ATmega328P with a Altera CPLD as a logic analyser and wrtiting a Cypress 1Mbit SRAM.
 Plug in the USB and then run the logic_analyser_extract.exe program to transfer the data to your computer.
  
 Remember to change your fuse bits so the ATmega328P uses a 20MHz crystal
 avrdude -p atmega328p -c usbtiny -U lfuse:w:0xde:m -U hfuse:w:0xd9:m -U efuse:w:0xff:m
 
 With clock output
 avrdude -p atmega328p -c usbtiny -U lfuse:w:0x9e:m -U hfuse:w:0xd9:m -U efuse:w:0xff:m 
 
 */

// ATmega328 Pin Map
//
//                           +-\/-+
//               Reset PC6  1|o   |28  PC5 Read byte 5
//          CPLD Reset PD0  2|    |27  PC4 Read byte 4
//  CPLD Start Capture PD1  3|    |26  PC3 Read byte 3
//              USB D+ PD2  4|    |25  PC2 Read byte 2
//              USB D- PD3  5|    |24  PC1 Read byte 1
//                     PD4  6|    |23  PC0 Read byte 0
//                     VCC  7|    |22  GND
//                     GND  8|    |21  AREF
//                XTAL PB6  9|    |20  AVCC
//                XTAL PB7 10|    |19  PB5 SCK
//       Analog Switch PD5 11|    |18  PB4 MISO
//         Read byte 6 PD6 12|    |17  PB3 MOSI
//         Read byte 7 PD7 13|    |16  PB2 CPLD Writing Pin
//       Clock output  PB0 14|    |15  PB1 CPLD Reading Next Address
//                           +----+

#define F_CPU 20000000 // If this will be changed, modify it in usbconfig.h too

#define cpldresetPin PD0
#define startcapturePin PD1
#define analogswitchPin PD5
#define nextaddressPin PB1
#define readingPin PB2

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"

uint8_t inusbTransfer = true;
uint32_t counting = 0;
uint32_t endCount = 131000; // SRAM end (approx as the CPLD captured 50 data points before the start of the trigger)

// V-USB Transfer variables
#define USB_TRANSFER 1
static uchar replyBuf[254];
static uchar *nextDigit;
static uchar dataReceived = 0, dataLength = 0; // for USB_DATA_IN

// This gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	if (rq->bRequest == USB_TRANSFER) { // custom command is in the bRequest field
		nextDigit = &replyBuf[-1];
		
		// Loop 125 (x2 bytes) times to send this data over USB
		for (int a = 0; a < 125; a++) {
			PORTB |= (1<<nextaddressPin); // Tell the CPLD we want the next address
			asm ("nop"); // Wait a little bit for CPLD/SRAM to show proper reading
			asm ("nop");
			
			// Read PD7-6 & PC6-0 and convert the two 4 bytes to hex
			uint8_t lowbyteReading = (PINC & 0x0F);
			uint8_t highbyteReading = ((PIND & 0xC0) | (PINC & 0x30)) >> 4;
			if (lowbyteReading < 10) {
				lowbyteReading = lowbyteReading + 48;
			}
			else {
				lowbyteReading = 'A' + lowbyteReading - 10;
			}
			if (highbyteReading < 10) {
				highbyteReading = highbyteReading + 48;
			}
			else {
				highbyteReading = 'A' + highbyteReading - 10;
			}
			*++nextDigit = highbyteReading; // Add both bytes to reply buffer
			*++nextDigit = lowbyteReading;
			
			PORTB &= ~(1<<nextaddressPin); 
			
			// Count until the end of the SRAM
			if (counting >= endCount) {
				inusbTransfer = false;
			}
			counting++; 
		}
		*++nextDigit = '\0';
		
		usbMsgPtr = replyBuf;
		return sizeof(replyBuf);
	}
	return 0;
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {
	uchar i;
	
	for (i = 0; dataReceived < dataLength && i < len; i++, dataReceived++) {
		replyBuf[dataReceived] = data[i];
	}
	
	return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}

// Transfers SRAM via USB
void transferData (void) {
	// Reset variables
	inusbTransfer = true;
	counting = 0;
	
	while (inusbTransfer == true) {
		usbPoll();
	}
}

int main (void) {
	setup();
	
	// Wait for CPLD clock to stablise
	_delay_ms(1000);
	
	// Change analog switch to 50MHz clock
	PORTD &= ~(1<<analogswitchPin);
	
	// Tell CPLD to start capturing data
	PORTB &= ~(1<<readingPin);
	PORTD |= (1<<cpldresetPin);
	_delay_us(20); // Simulate waiting for a trigger
	
	// Trigger the CPLD to write the start address of our capture
	PORTD |= (1<<startcapturePin);
	
	// Wait 10ms for the capture
	_delay_ms(10);
	
	// Finish writing
	PORTB |= (1<<readingPin);
	PORTD &= ~(1<<startcapturePin);
	_delay_us(50);
	
	// Change analog switch to slower clock
	PORTD |= (1<<analogswitchPin);
	_delay_us(50);
	
	// For testing re-reading SRAM
	/*while (1) {
		for (int x = 0; x < 1000; x++) {
			PORTB |= (1<<nextaddressPin); // Tell the CPLD we want the next address
			asm ("nop");
			asm ("nop");
			asm ("nop");
			PORTB &= ~(1<<nextaddressPin);
			asm ("nop");
			asm ("nop");
			asm ("nop");
		}
		
		PORTD &= ~(1<<cpldresetPin);
		asm ("nop");
		asm ("nop");
		asm ("nop");
		PORTD |= (1<<cpldresetPin);
	}*/
	
	// Transfer data
	startUSB();
	transferData();
	usbDeviceDisconnect();
	
	while(1);
	
	return 0;
}
